from datetime import timedelta
import os, sys
import django



def setup_django():
    from django.conf import settings

    # USE_L10N is deprecated, and will be removed in Django 5.0.
    use_l10n = {"USE_L10N": True} if django.VERSION < (4, 0) else {}

    settings.configure(
        DEBUG_PROPAGATE_EXCEPTIONS=True,
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:'
            },
        },
        SECRET_KEY='not very secret in tests',
        USE_I18N = True,
        ROOT_URLCONF='tests.urls',
        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'APP_DIRS': True,
                'OPTIONS': {
                    "debug": True,  # We want template errors to raise
                }
            },
        ],
        MIDDLEWARE=(
            'django.middleware.common.CommonMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
        ),
        INSTALLED_APPS=(
            'django.contrib.admin',
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sites',
            'django.contrib.staticfiles',
            'rest_framework',
            'tests',
            'tests.sample',
            'django_jwt_extended',   
        ),
        JWT_CONFIG = {
            'ALGORITHM': 'HS256',
            'LOCATION': ['headers', 'cookies'],
            'ACCESS_TOKEN_EXPIRES': timedelta(days=2),
            'REFRESH_TOKEN_EXPIRES': timedelta(days=30),
            'JWT_NOT_FOUND_MSG': {'msg': "can't find JWT token."},
            'ACCESS_TOKEN_COOKIE_NAME': 'access_token',
            'REFRESH_TOKEN_COOKIE_NAME': 'refresh_token'
        },
        TIME_ZONE='Asia/Seoul',
        USE_TZ=False,
        **use_l10n,
    )

    django.setup()

setup_django()


import unittest
from datetime import timedelta
from django_jwt_extended.config import ConfigParser
from django_jwt_extended.exceptions import (
    InvalidJwtAlgorithm,
    InvalidLocation,
    InvalidExpires,
)
from django.apps import apps


class ConfigParserTestCase(unittest.TestCase):

    def setUp(self):
        self.config_parser = ConfigParser

    def tearDown(self):
        pass

    def test_valid_jwt_algorithm(self):
        """Validate JWT algorithm config"""
        self.config_parser.validate_jwt_algorithm({})
        self.config_parser.validate_jwt_algorithm(
            {'ALGORITHM': 'HS256'}
        )
        try:
            self.config_parser.validate_jwt_algorithm(
                {'ALGORITHM': 'BAD_ALGO'}
            )
            self.assertFalse(True)
        except InvalidJwtAlgorithm:
            pass
        
    def test_valid_token_location(self):
        """Validate JWT Token location config"""
        self.config_parser.validate_token_location({})
        self.config_parser.validate_token_location(
            {'LOCATION': ['headers']}
        )
        try:
            self.config_parser.validate_token_location(
                {'LOCATION': ['header']})
            self.config_parser.validate_token_location(
                {'LOCATION': 'header'})
            self.assertFalse(True)
        except InvalidLocation:
            pass
        
    def test_valid_token_expires(self):
        """Validate Token Expires config"""
        cases = [
            (
                self.config_parser.validate_access_token_expires, 
                'ACCESS_TOKEN_EXPIRES'
            ),
            (
                self.config_parser.validate_refresh_token_expires, 
                'REFRESH_TOKEN_EXPIRES'
            ),
        ]
        for module, key, in cases:
            module({})
            module({key: 100})
            module({key: timedelta(days=1)})
            try:
                module({key: -1})
                module({key: "INVALID"})
                self.assertFalse(True)
            except InvalidExpires:
                pass
        

if __name__ == '__main__':
    unittest.main()


import json
from datetime import timedelta
from django.utils import timezone
from django.apps import apps
from .exceptions import (
    ConfigIsNotDict,
    InvalidJsonFormat,
    InvalidJwtAlgorithm,
    InvalidLocation,
    InvalidExpires,
)

allowed_algorithm = ('HS256',)
allowed_location = ('headers', 'cookies',)

class ConfigParser:

    def __init__(self, config: dict):
        if not isinstance(config, dict):
            raise ConfigIsNotDict()
        self.access_token_cookie_name = self.validate_access_token_cookie_name(config)
        self.refresh_token_cookie_name = self.validate_refresh_token_cookie_name(config)
        self.jwt_algorithm = self.validate_jwt_algorithm(config)
        self.token_location = self.validate_token_location(config)
        self.access_token_expires = self.validate_access_token_expires(config)
        self.refresh_token_expires = self.validate_refresh_token_expires(config)
        self.errors = self.customize_error(config)

    @staticmethod
    def validate_access_token_cookie_name(config: dict):
        access_cookie_name = config.get('ACCESS_TOKEN_COOKIE_NAME', 'access_token')
        return access_cookie_name
    
    @staticmethod
    def validate_refresh_token_cookie_name(config: dict):
        refresh_cookie_name = config.get('REFRESH_TOKEN_COOKIE_NAME', 'refresh_token')
        return refresh_cookie_name

    @staticmethod
    def validate_jwt_algorithm(config: dict):
        jwt_algorithm = config.get('ALGORITHM', 'HS256')
        if jwt_algorithm not in allowed_algorithm:
            raise InvalidJwtAlgorithm(jwt_algorithm, allowed_algorithm)
        return jwt_algorithm

    @staticmethod
    def validate_token_location(config: dict):
        token_location = config.get('LOCATION', ['headers', 'cookies'])
        if not (
            isinstance(token_location, list)
            and not (set(token_location) - set(allowed_location))
        ):
            raise InvalidLocation(token_location, allowed_location)
        return token_location

    @staticmethod
    def validate_access_token_expires(config: dict):
        expires = config.get('ACCESS_TOKEN_EXPIRES', timedelta(days=2))
        if isinstance(expires, int) and expires > 0:
            expires = timedelta(seconds=expires)
        if not isinstance(expires, timedelta):
            raise InvalidExpires('ACCESS_TOKEN')
        return expires

    @staticmethod
    def validate_refresh_token_expires(config: dict):
        expires = config.get('REFRESH_TOKEN_EXPIRES', timedelta(days=30))
        if isinstance(expires, int) and expires > 0:
            expires = timedelta(seconds=expires)
        if not isinstance(expires, timedelta):
            raise InvalidExpires('REFRESH_TOKEN')
        return expires

    @staticmethod
    def customize_error(config: dict):

        def validate_json(data: dict):
            try:
                json.dumps(data)
            except ValueError:
                return False
            return True

        default_error = {
            'JWT_NOT_FOUND_MSG': {'msg': 'JWT token not found'},
            'DECODE_ERROR_MSG': {'msg': 'Signature verification failed.'},
            'EXPIRED_TOKEN_MSG': {'msg': 'JWT token has expired'},
            'INVALID_TOKEN_TYPE_MSG': {'msg': "Invalid JWT token type"},
            'INVALID_NBF_MSG': {'msg': "The token is not yet valid (nbf)"},
            'BEARER_ERROR_MSG': {
                'msg':(
                        f"Missing 'Bearer' type in "
                        f"'Authorization' header."
                        f" Expected 'Authorization: "
                        f"Bearer <JWT>'"
                    )
            },
        }

        customized_error = {}
        for error in default_error.keys():
            target = config.get(error, default_error[error])
            if not validate_json(target):
                raise InvalidJsonFormat()

            customized_error[error] = target

        return customized_error



from datetime import timedelta
from django.apps import AppConfig
from django.conf import settings
from .config import ConfigParser
from .exceptions import NotFoundSecretKey


class DjangoJwtExtConfig(AppConfig):
    name = 'django_jwt_extended'
    verbose_name = "Django JWT Extended"

    def ready(self):
        if not hasattr(settings, 'SECRET_KEY'):
            raise NotFoundSecretKey()

        data = ConfigParser(
            settings.JWT_CONFIG
            if hasattr(settings, 'JWT_CONFIG')
            else {}
        )
        self.jwt_algorithm = data.jwt_algorithm
        self.token_location = data.token_location
        self.access_token_expires = data.access_token_expires
        self.refresh_token_expires = data.refresh_token_expires
        self.token_header_name = 'Authorization'
        self.access_token_cookie_name = data.access_token_cookie_name
        self.refresh_token_cookie_name = data.refresh_token_cookie_name

        self.jwt_not_found_msg = data.errors['JWT_NOT_FOUND_MSG']
        self.bearer_error_msg = data.errors['BEARER_ERROR_MSG']
        self.decode_error_msg = data.errors['DECODE_ERROR_MSG']
        self.expired_token_msg = data.errors['EXPIRED_TOKEN_MSG']
        self.invalid_token_type_msg = data.errors['INVALID_TOKEN_TYPE_MSG']
        self.invalid_nbf_msg = data.errors['INVALID_NBF_MSG']
        


class NotFoundRequest(Exception):

    def __init__(self, fn_name: str):
        self.fn_name = fn_name

    def __str__(self):
        return (
            f'In "{self.fn_name}" view, "Request object" was not detected in the argument.',
            'If you run into any issues, please report them here: (https://github.com/iml1111/django-jwt-extended/issues)'
        )


class NotFoundSecretKey(Exception):
    def __str__(self):
        return f'In Settings, "SECRET_KEY" was not detected.'


class ConfigIsNotDict(Exception):
    def __str__(self):
        return '"JWT_EXTENDED" config must be "dict" type.'


class InvalidJwtAlgorithm(Exception):

    def __init__(self, param: str, algorithms):
        self.param = param
        self.algorithms = algorithms

    def __str__(self):
        return (
            f'Invalid JWT Algorithm "{self.param}". '
            f'ALGORITHM must be in {self.algorithms}.'
        )


class InvalidLocation(Exception):

    def __init__(self, param: str, locations):
        self.param = param
        self.locations = locations

    def __str__(self):
        return (
            f'Invalid Location "{self.param}". '
            f'LOCATION must be "list", in {self.locations}.'
        )


class InvalidExpires(Exception):

    def __init__(self, target: str):
        self.target = target

    def __str__(self):
        return (
            f'Invalid {self.target} Expires. '
            f'EXPIRES must be a "timedelta" or "int"(value > 0)'
        )


class InvalidRequest(Exception):

    def __init__(self, param: str):
        self.param = param

    def __str__(self):
        return (
            f"Missing Django Request Object."
            f", not {self.param}"
        )


class InvalidOptional(Exception):

    def __init__(self, param: str):
        self.param = param

    def __str__(self):
        return (
            f"'optional' param must be bool type, not {self.param}."
        )


class InvalidRefresh(Exception):

    def __init__(self, param: str):
        self.param = param

    def __str__(self):
        return (
            f"'refresh' param must be bool type, not {self.param}."
        )


class InvalidJsonFormat(Exception):

    def __str__(self):
        return (
            f"Invalid JSON format."
            f"Error config must be JSON serializable."
        )


from datetime import datetime
from uuid import uuid4
import jwt
from django.apps import apps
from django.http import HttpRequest
from django.conf import settings
from .apps import DjangoJwtExtConfig
from .request import REQUESTS
from .exceptions import InvalidRequest


def create_access_token(identity):
    config = apps.get_app_config('django_jwt_extended')
    payload = _create_payload(identity, 'access', config)
    return jwt.encode(
        payload=payload,
        key=settings.SECRET_KEY,
        algorithm=config.jwt_algorithm
    )


def create_refresh_token(identity):
    config = apps.get_app_config('django_jwt_extended')
    payload = _create_payload(identity, 'refresh', config)
    return jwt.encode(
        payload=payload,
        key=settings.SECRET_KEY,
        algorithm=config.jwt_algorithm
    )


def get_jwt_identity(request: HttpRequest):
    if not any([isinstance(request, obj) for obj in REQUESTS]):
        raise InvalidRequest(str(type(request)))
    if 'jwt_payload' in request.META:
        return request.META['jwt_payload'].get('sub')
    else:
        return None


def get_jwt(request: HttpRequest):
    if not any([isinstance(request, obj) for obj in REQUESTS]):
        raise InvalidRequest(str(type(request)))
    return request.META.get('jwt_payload')


"""Inner Func"""
def _create_payload(identity, type: str, config: DjangoJwtExtConfig):
    if type == 'access':
        expires = config.access_token_expires
    else:
        expires = config.refresh_token_expires
    now = datetime.utcnow()
    return {
        'iat': now,
        'jti': str(uuid4()),
        'type': type,
        'sub': identity,
        'nbf': now,
        'exp': now + expires,
    }


def _find_jwt_token(request, refresh: bool, config: DjangoJwtExtConfig):
    for location in config.token_location:
        if (
            location == 'headers'
            and config.token_header_name in request.headers
        ):
            return request.headers[config.token_header_name], location

        elif location == 'cookies':
            if not refresh and config.access_token_cookie_name in request.COOKIES:
                return request.COOKIES[config.access_token_cookie_name], location
            elif refresh and config.refresh_token_cookie_name in request.COOKIES:
                return request.COOKIES[config.refresh_token_cookie_name], location

    return None, None


def _parse_jwt_token(jwt_token: str, location: str):
    if location == 'headers':
        if jwt_token.startswith('Bearer '):
            return jwt_token[7:]
    elif location == 'cookies':
        return jwt_token
    else:
        return None


def _validate_payload(payload: dict, type: str):
    if payload['type'] != type:
        return 'invalid type'
    else:
        return "valid"


from http.cookiejar import Cookie
import unittest, json, jwt
from django.conf import settings
from django.test import RequestFactory
from django.apps import apps
from tests.sample.views import (
    login, decorator_user, refresh, 
    user, user_optional,
    RestAPIView, rest_user
)


class AuthTestCase(unittest.TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.access_token, self.refresh_token = self._get_tokens()
        self.methods = (
            self.factory.get,
            self.factory.post,
            self.factory.put,
            self.factory.delete,
        )

    def tearDown(self):
        pass

    def test_auth_basic(self):
        """Run Authentication basic"""
        request = self.factory.get(
            '/user',
            HTTP_Authorization=(
                "Bearer " + self.access_token
            )
        )
        response = user(request)
        self.assertEqual(response.status_code, 200)

    def test_auth_decorator(self):
        """Run Authentication basic with Decorator"""
        request = self.factory.get(
            '/user',
            HTTP_Authorization=(
                "Bearer " + self.access_token
            )
        )
        response = decorator_user(request)
        self.assertEqual(response.status_code, 200)

    def test_refresh(self):
        """Run Token Refresh & reauth"""
        request = self.factory.get(
            '/refresh',
            HTTP_Authorization=(
                "Bearer "
                + self.refresh_token
            )
        )
        response = refresh(request)
        self.assertEqual(response.status_code, 200)
        tokens = json.loads(response.content)
        access_token = tokens['access_token']

        request = self.factory.get(
            '/user',
            HTTP_Authorization=(
                "Bearer "
                + access_token
            )
        )
        response = user(request)
        self.assertEqual(response.status_code, 200)

    def test_auth_optional(self):
        """Run Auth Optional"""
        auth_request = self.factory.get(
            '/user_optional',
            HTTP_Authorization=(
                "Bearer " + self.access_token
            )
        )
        no_auth_request = self.factory.get('/user_optional')

        response = user_optional(auth_request)
        self.assertEqual(response.status_code, 200)
        response = user_optional(no_auth_request)
        self.assertEqual(response.status_code, 200)

    def test_rest_api_class(self):
        """Run REST API View Class Auth"""
        view = RestAPIView.as_view()

        path = "/rest-api-class"
        token = "Bearer " + self.access_token
        for method in self.methods:
            request = method(path, HTTP_Authorization=token)
            response = view(request)
            self.assertEqual(response.status_code, 200)

    def test_rest_api_func(self):
        """Run REST API View Func Auth"""
        path = "/rest-api-func"
        token = "Bearer " + self.access_token
        for method in self.methods:
            request = method(path, HTTP_Authorization=token)
            response = rest_user(request, "hello")
            self.assertEqual(response.status_code, 200)

    def test_invalid_token_auth(self):
        """Run Invalid Token Auth"""
        payload = {'sub': 'invalid'}
        algorithm = 'HS256'
        secret_key = "invalid"
        invalid_token = jwt.encode(payload, secret_key, algorithm)

        request = self.factory.get(
            '/user',
            HTTP_Authorization=(
                "Bearer " + invalid_token
            )
        )
        response = user(request)
        self.assertEqual(response.status_code, 401)

    def test_token_not_found(self):
        """Test No Input Token Auth"""
        request = self.factory.get('/user')
        response = user(request)
        self.assertEqual(response.status_code, 401)
        self.assertEqual(
            response.content.decode(encoding='utf8'),
            json.dumps({'msg': "can't find JWT token."})
        )

    def test_token_header_not_bearer(self):
        """Test Token Header Not Bearer"""
        request = self.factory.get(
            '/user',
            HTTP_Authorization=(
                self.access_token
            )
        )
        response = user(request)
        self.assertEqual(response.status_code, 401)

    def test_expired_token_auth(self):
        """Test Expired Token Auth"""
        payload = {'sub': 'iml', 'exp': 1}
        algorithm = 'HS256'
        secret_key = settings.SECRET_KEY
        exp_token = jwt.encode(payload, secret_key, algorithm)

        request = self.factory.get(
            '/user',
            HTTP_Authorization=(
                "Bearer " + exp_token
            )
        )
        response = user(request)
        self.assertEqual(response.status_code, 401)

    #cookie test
    def test_auth_basic_cookie(self):
        """Run Authentication basic"""
        request = self.factory.get('/user')
        request.COOKIES = {'access_token': self.access_token}
        response = user(request)
        self.assertEqual(response.status_code, 200)

    def test_auth_decorator_cookie(self):
        """Run Authentication basic with Decorator"""
        request = self.factory.get('/user')
        request.COOKIES = {'access_token': self.access_token}
        response = decorator_user(request)
        self.assertEqual(response.status_code, 200)

    def test_refresh_cookie(self):
        """Run Token Refresh & reauth"""
        request = self.factory.get('/refresh')
        request.COOKIES = {'refresh_token': self.refresh_token}
        response = refresh(request)
        self.assertEqual(response.status_code, 200)
        tokens = json.loads(response.content)
        access_token = tokens['access_token']

        request = self.factory.get('/user')
        request.COOKIES = {'access_token': access_token}
        response = user(request)
        self.assertEqual(response.status_code, 200)

    def test_auth_optional_cookie(self):
        """Run Auth Optional"""
        auth_request = self.factory.get('/user_optional')
        auth_request.COOKIES = {'access_token': self.access_token}
        no_auth_request = self.factory.get('/user_optional')
        no_auth_request.COOKIES = {'access_token': self.access_token}

        response = user_optional(auth_request)
        self.assertEqual(response.status_code, 200)
        response = user_optional(no_auth_request)
        self.assertEqual(response.status_code, 200)

    def test_rest_api_class_cookie(self):
        """Run REST API View Class Auth"""
        view = RestAPIView.as_view()

        path = "/rest-api-class"
        token = {'access_token': self.access_token}
        for method in self.methods:
            request = method(path)
            request.COOKIES = token
            response = view(request)
            self.assertEqual(response.status_code, 200)

    def test_rest_api_func_cookie(self):
        """Run REST API View Func Auth"""
        path = "/rest-api-func"
        token = {'access_token': self.access_token}
        for method in self.methods:
            request = method(path)
            request.COOKIES = token
            response = rest_user(request, "hello")
            self.assertEqual(response.status_code, 200)

    def test_invalid_token_auth_cookie(self):
        """Run Invalid Token Auth"""
        payload = {'sub': 'invalid'}
        algorithm = 'HS256'
        secret_key = "invalid"
        invalid_token = jwt.encode(payload, secret_key, algorithm)

        request = self.factory.get('/user')
        request.COOKIES = {'access_token': invalid_token}
        response = user(request)
        self.assertEqual(response.status_code, 401)

    def test_token_not_found_cookie(self):
        """Test No Input Token Auth"""
        request = self.factory.get('/user')
        response = user(request)
        self.assertEqual(response.status_code, 401)
        self.assertEqual(
            response.content.decode(encoding='utf8'),
            json.dumps({'msg': "can't find JWT token."})
        )

    def test_expired_token_auth_cookie(self):
        """Test Expired Token Auth"""
        payload = {'sub': 'iml', 'exp': 1}
        algorithm = 'HS256'
        secret_key = settings.SECRET_KEY
        exp_token = jwt.encode(payload, secret_key, algorithm)

        request = self.factory.get('/user')
        request.COOKIE = {'access_token': exp_token}
        response = user(request)
        self.assertEqual(response.status_code, 401)

    def _get_tokens(self):
        request = self.factory.get('/login')
        response = login(request)
        self.assertEqual(response.status_code, 200)

        tokens = json.loads(response.content)
        access_token = tokens['access_token']
        refresh_token = tokens['refresh_token']
        return access_token, refresh_token


if __name__ == '__main__':
    unittest.main()


from django.http import JsonResponse
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.decorators import api_view
from django_jwt_extended import jwt_required
from django_jwt_extended import create_access_token
from django_jwt_extended import create_refresh_token
from django_jwt_extended import get_jwt_identity
from django_jwt_extended import get_jwt


# Login and issue tokens
def login(request):
    return JsonResponse({
        "access_token": create_access_token("iml"),
        'refresh_token': create_refresh_token('iml'),
    })

# Refresh tokens
@jwt_required(refresh=True)
def refresh(request):
    identity = get_jwt_identity(request)
    return JsonResponse({
        "access_token": create_access_token(identity),
        'refresh_token': create_refresh_token(identity),
    })


# Authentication access token
@jwt_required()
def user(request):
    identity = get_jwt_identity(request)
    payload = get_jwt(request)
    return JsonResponse({
        'id': identity,
        'raw_jwt': payload,
    })

# Authentication access token with Decorator
def login_required(func):
    @jwt_required()
    def wrapper(request, **path):
        identity = get_jwt_identity(request)
        request.META['logined_identity'] = identity
        return func(request, **path)
    return wrapper


@login_required
def decorator_user(request):
    identity = request.META['logined_identity']
    payload = get_jwt(request)

    return JsonResponse({
        'id': identity,
        'raw_jwt': payload,
    })


# Optional Login test
@jwt_required(optional=True)
def user_optional(request):
    identity = get_jwt_identity(request)
    payload = get_jwt(request)
    return JsonResponse({
        'id': identity,
        'raw_jwt': payload,
    })


# Rest framework test
class RestAPIView(APIView):

    @jwt_required()
    def get(self, request):
        identity = get_jwt_identity(request)
        payload = get_jwt(request)
        return Response({
            'id': identity,
            'raw_jwt': payload,
        })

    @jwt_required()
    def post(self, request):
        identity = get_jwt_identity(request)
        payload = get_jwt(request)
        return Response({
            'id': identity,
            'raw_jwt': payload,
        })

    @jwt_required()
    def put(self, request):
        identity = get_jwt_identity(request)
        payload = get_jwt(request)
        return Response({
            'id': identity,
            'raw_jwt': payload,
        })

    @jwt_required()
    def delete(self, request):
        identity = get_jwt_identity(request)
        payload = get_jwt(request)
        return Response({
            'id': identity,
            'raw_jwt': payload,
        })


# Rest framework Func test
@api_view(['GET', 'POST', 'PUT', 'DELETE'])
@jwt_required()
def rest_user(request, hello: str):
    identity = get_jwt_identity(request)
    payload = get_jwt(request)
    return Response({
        'id': identity,
        'raw_jwt': payload,
        'hello': hello,
    })


import django
from .decorators import jwt_required
from .tokens import (
    create_access_token,
    create_refresh_token,
    get_jwt,
    get_jwt_identity,
)

__AUTHOR__ = "IML"
__VERSION__ = "1.0.0"

default_app_config = 'django_jwt_extended.apps.DjangoJwtExtConfig'





from functools import wraps
from django.http import JsonResponse
from django.conf import settings
from django.apps import apps
from .request import _find_request_object
from .tokens import _find_jwt_token, _parse_jwt_token, _validate_payload
from .exceptions import (
	NotFoundRequest,
	InvalidOptional,
	InvalidRefresh,
)
import jwt
from jwt.exceptions import (
	InvalidSignatureError,
	ImmatureSignatureError,
	ExpiredSignatureError,
)


def jwt_required(optional=False, refresh=False):
	"""View decorator"""
	if not isinstance(optional, bool):
		raise InvalidOptional(str(type(optional)))
	if not isinstance(refresh, bool):
		raise InvalidRefresh(str(type(refresh)))

	def wrapper(fn):
		@wraps(fn)
		def decorator(*args, **kwargs):
			""""do something"""
			request = _find_request_object(*args, **kwargs)

			# Django Request 객체를 찾을 수 없을 경우
			if request is None:
				raise NotFoundRequest(fn.__name__)
			config = apps.get_app_config('django_jwt_extended')
			jwt_token, location = _find_jwt_token(request, refresh, config)

			# 토큰을 찾을 수 없을 경우
			if not optional and jwt_token is None:
				return JsonResponse(config.jwt_not_found_msg, status=401)
			# 토큰을 찾을 수 없지만, optional인 경우
			elif optional and jwt_token is None:
				return fn(*args, **kwargs)

			# header 토큰에 한하여, Bearer 포맷이 아닐 경우
			jwt_token = _parse_jwt_token(jwt_token, location)
			if jwt_token is None:
				return JsonResponse(config.bearer_error_msg, status=401)

			try:
				payload = jwt.decode(
					jwt_token, settings.SECRET_KEY,
					config.jwt_algorithm,
				)
			except InvalidSignatureError:
				return JsonResponse(config.decode_error_msg, status=401)
			except ImmatureSignatureError:
				return JsonResponse(config.invalid_nbf_msg, status=401)
			except ExpiredSignatureError:
				return JsonResponse(config.expired_token_msg, status=401)


			# 토큰의 유효기간, 액세스/리프레시 검증
			valid = _validate_payload(payload, 'refresh' if refresh else 'access')
			if valid == 'invalid type':
				return JsonResponse(config.invalid_token_type_msg, status=401)

			request.META['jwt_payload'] = payload
			return fn(*args, **kwargs)
		return decorator
	return wrapper




"""
django-jwt-extended
-------------
An open source Django extension that provides Simple JWT Authentication
"""

from setuptools import setup
from django_jwt_extended import __VERSION__


with open("README.md", "r", encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='django-jwt-extended',
    version=__VERSION__,
    description='An open source Django extension that provides Simple JWT Authentication.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/iml1111/django-jwt-extended',
    author='IML',
    author_email='shin10256@gmail.com',
    license='MIT',
    keywords='django jwt extended',
    packages=['django_jwt_extended'],
    install_requires=['django'],
    platforms='any',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)


import unittest
from django.apps import apps

class BasicsTestCase(unittest.TestCase):

    def setUp(self):
        self.config = apps.get_app_config('django_jwt_extended')

    def tearDown(self):
        pass

    def test_module_exists(self):
        """Run Module Exists"""
        self.assertFalse(self.config is None)

    def test_config_exists(self):
        """App config Exists"""
        keys = (
            'jwt_algorithm',
            'token_location',
            'access_token_expires',
            'refresh_token_expires',
            'token_header_name',
            #error message
            'jwt_not_found_msg',
            'bearer_error_msg',
            'decode_error_msg',
            'expired_token_msg',
            'invalid_token_type_msg',
            'invalid_nbf_msg',
        )
        for key in keys:
            self.assertTrue(hasattr(self.config, key))

if __name__ == '__main__':
    unittest.main()